@@ -24,16 +24,11 @@
 /* audio-play.c - play files using AudioPlay GObject
  */
 
+#include "config.h"
+
 #include <gst/gst.h>
 #include "audio-play.h"
 
-#define DEBUG
-/* #define DEBUG_XML */
-#ifdef DEBUG_XML
-#include <stdio.h>
-FILE *_debug_file;
-#endif
-
 /* private object properties */
 struct AudioPlayPriv {
 	GstElement *pipeline;		/* main playback pipeline */
@@ -56,8 +51,11 @@ struct AudioPlayPriv {
 	gint seconds;			/* current playing time */
 	guint64 nanoseconds;
 
+	AudioPlayErrorHandler error_handler;
+	gpointer error_data;
 	guint idler_id;
 	guint iterator_id;
+	guint tick_timeout_id;
 };
 
 /* signal enum */
@@ -70,32 +68,35 @@ enum {
 
 /* GError enum */
 enum {
+	AUDIO_PLAY_ERROR_SRC,
+	AUDIO_PLAY_ERROR_TYPEFIND,
 	AUDIO_PLAY_ERROR_AUTOPLUG
 };
 
 static guint audio_play_signals [LAST_SIGNAL] = { 0 };
-	
+
 
 /* GError quark stuff */
-GQuark
+static GQuark
 audio_play_error_quark (void)
 {
 	static GQuark quark = 0;
-	if (quark == 0) 
+	if (quark == 0)
 		quark = g_quark_from_static_string ("audio-play-error-quark");
 	return quark;
 }
 
 /* forward declarations */
-static void	audio_play_class_init 	(AudioPlayClass *klass);
-static void	audio_play_init 	(AudioPlay *play);
+static void	audio_play_class_init	(AudioPlayClass *klass);
+static void	audio_play_init		(AudioPlay *play);
 
 static gboolean	audio_play_playing_to_ready (AudioPlay *play);
-//static void	gst_play_dispose	(GObject *object);
+static void	audio_play_dispose	(GObject *object);
+static void	audio_play_finalize	(GObject *object);
+static void	callback_sink_eos 	(GstElement *element, AudioPlay *play);
 
 /* GObject type stuff */
-
-static GstElementClass *parent_class = NULL;
+static GObject *parent_class = NULL;
 
 GType
 audio_play_get_type (void)
@@ -113,8 +114,8 @@ audio_play_get_type (void)
 			0, (GInstanceInitFunc) audio_play_init,
 			NULL
 		};
-		play_type = g_type_register_static (G_TYPE_OBJECT, "AudioPlay", 
-			                    	    &play_info, 0);
+		play_type = g_type_register_static (G_TYPE_OBJECT, "AudioPlay",
+						    &play_info, 0);
 										        }
 	return play_type;
 }
@@ -122,20 +123,111 @@ audio_play_get_type (void)
 
 /* private functions */
 /* the lazy man's version of debugging */
-void
+static void
 audio_play_debug (AudioPlay *play)
 {
-	g_print ("DEBUG: audio_play %p\n", play);
-	g_print ("DEBUG: pipeline %p\n", play->priv->pipeline);
-	g_print ("DEBUG: typefind %p\n", play->priv->typefind);
-	g_print ("DEBUG: decoder %p\n", play->priv->decoder);
-	g_print ("DEBUG: time %d\n", play->priv->seconds);
+	NM_DEBUG("audio_play %p\n", play);
+	NM_DEBUG("pipeline %p\n", play->priv->pipeline);
+	NM_DEBUG("typefind %p\n", play->priv->typefind);
+	NM_DEBUG("decoder %p\n", play->priv->decoder);
+	NM_DEBUG("time %d\n", play->priv->seconds);
 }
 
+/* error function; calls the handler if it's there */
+
+static void
+audio_play_error (AudioPlay *play, const gchar *message)
+{
+	if (play == NULL)
+	{
+		g_warning ("No play object passed.");
+		return;
+	}
+	g_print ("audio-play error: %s\n", message);
+	if (play->priv->error_handler)
+		play->priv->error_handler (message, play->priv->error_data);
+}
+/* handler functions dispatched from iterator */
+
+static void
+handle_have_type (AudioPlay *play)
+{
+	AudioPlayPriv *priv = play->priv;
+	GstCaps *caps = priv->caps;
+	gchar *mime = g_strdup_printf (gst_caps_get_mime (caps));
+	GstElement *decoder = NULL;
+
+#ifdef DEBUG
+	fprintf (stderr, "handle_have_type\n");
+	fprintf (stderr, "have caps, mime type is %s\n", mime);
+#endif
+
+	g_assert (GST_IS_ELEMENT (priv->pipeline));
+	priv->have_type = FALSE;
+
+	gst_element_set_state (priv->pipeline, GST_STATE_READY);
+
+	/* disconnect the typefind from the pipeline and remove it,
+	 * since we now know what the type is */
+	NM_DEBUG("disconnecting typefind\n");
+	gst_element_disconnect (priv->src, priv->typefind);
+	/* ref it so it doesn't go away */
+	g_object_ref (priv->typefind);
+	NM_DEBUG("removing typefind\n");
+	gst_bin_remove (GST_BIN (priv->pipeline), priv->typefind);
+
+	/* gst_scheduler_show (GST_ELEMENT_SCHED (priv->pipeline)); */
+
+	/* now based on the mime type set up the pipeline properly */
+	NM_DEBUG("deciding on decoder\n");
+	if (strstr (mime, "mp3"))
+		decoder = gst_element_factory_make ("mad", "decoder");
+	else if (strstr (mime, "x-ogg"))
+		decoder = gst_element_factory_make ("vorbisfile", "decoder");
+	else if (strstr (mime, "x-wav"))
+		decoder = gst_element_factory_make ("wavparse", "decoder");
+	else if (strstr (mime, "x-flac"))
+		decoder = gst_element_factory_make ("flacdec", "decoder");
+	else if (strstr (mime, "x-mod") || strstr (mime, "x-s3m") || 
+	         strstr (mime, "x-xm") || strstr (mime, "x-it"))
+		decoder = gst_element_factory_make ("modplug", "decoder");
+	else
+	{
+		g_print ("mime type %s not handled in this program.\n", mime);
+		g_free (mime);
+		return;
+	}
+	g_free (mime);
+	priv->decoder = decoder;
+
+	/* set it to null */
+	/* somewhere between 0.4.2 and 0.5.0, setting this to NULL
+	 * broke threads completely because of the cothread freeing
+	 * or something to that effect
+	 * FIXME: figure out why that happened so we can set it to NULL
+	 * again */
+	gst_element_set_state (priv->pipeline, GST_STATE_NULL);
+
+	/* now put playback pipe in place of where typefind was  */
+	NM_DEBUG("putting in decoder\n");
+	gst_bin_add (GST_BIN (priv->pipeline), priv->decoder);
+	NM_DEBUG("putting in sink\n");
+	gst_bin_add (GST_BIN (priv->pipeline), priv->sink);
+	/* FIXME: this shouldn't go here */
+	g_signal_connect (G_OBJECT (play->priv->sink), "eos",
+			  G_CALLBACK (callback_sink_eos), play);
+	if (!gst_element_connect (priv->src, priv->decoder))
+		g_error ("warning: could not connect src and decoder !\n");
+	if (!gst_element_connect (priv->decoder, priv->sink))
+		g_print ("warning: could not connect decoder and sink !\n");
+	if (gst_element_set_state (priv->pipeline, GST_STATE_PAUSED)
+			== GST_STATE_FAILURE)
+		g_print ("pipelien could not be paused !\n");
+}
 
 /* callbacks & idlers */
-/* iterator idler func */
-gboolean
+/* iterator idler func; iterates pipeline */
+static gboolean
 iterator (AudioPlay *play)
 {
 #ifdef DEBUG
@@ -145,28 +237,30 @@ iterator (AudioPlay *play)
 		return TRUE;
 	else
 	{
-		g_print ("DEBUG: iterator: couldn't iterate\n");
+		NM_DEBUG("iterator: couldn't iterate\n");
 		/* do an eos */
 		play->priv->have_eos = TRUE;
 		return FALSE;
 	}
 }
 
-/* main idler func, handles all sorts of requests asynchronously */
-gboolean
+/* main idler func, handles requests and signal actions asynchronously */
+static gboolean
 idler (AudioPlay *play)
 {
 #ifdef DEBUG
 	g_print ("-");
 #endif
+	if (play->priv->have_type)
+		handle_have_type (play);
 	if (play->priv->have_eos)
 	{
-		g_print ("DEBUG: signalling eos\n");
+		NM_DEBUG("signalling eos\n");
 		play->priv->have_eos = FALSE;
 		audio_play_playing_to_ready (play);
 		/* gst_main_quit (); */
-		g_signal_emit (G_OBJECT (play), 
-			       audio_play_signals [EOS_SIGNAL], 
+		g_signal_emit (G_OBJECT (play),
+			       audio_play_signals [EOS_SIGNAL],
 			       0);
 		return FALSE;
 	}
@@ -174,7 +268,7 @@ idler (AudioPlay *play)
 		return TRUE;
 	else
 	{
-		g_print ("DEBUG: idler returning false, not playing\n");
+		NM_DEBUG("idler returning false, not playing\n");
 		//return FALSE;
 	}
 }
@@ -194,8 +288,8 @@ tick_timeout (AudioPlay *play)
 	{
 		priv->seconds = secs;
 		priv->nanoseconds = nanosecs;
-		g_signal_emit (G_OBJECT (play), 
-			       audio_play_signals [TICK_SIGNAL], 
+		g_signal_emit (G_OBJECT (play),
+			       audio_play_signals [TICK_SIGNAL],
 			       0, priv->nanoseconds);
 	}
 	/* remove ourselves when we stop playing */
@@ -203,104 +297,82 @@ tick_timeout (AudioPlay *play)
 }
 
 
-void
+static void
 callback_sink_eos (GstElement *element, AudioPlay *play)
 {
 	AudioPlayPriv *priv = play->priv;
 
-  	g_print ("DEBUG: sink_eos triggered\n");
+	NM_DEBUG("sink_eos triggered\n");
 	play->priv->have_eos = TRUE;
 	/* gst_main_quit (); */
 }
 
+/* FIXME: would you please sanitify naming of callbacks and so on ? */
+/* callback used for errors  coming from GStreamer */
+static void
+error_callback (GObject *object, GstObject *origin, gchar *error,
+		gpointer data)
+{
+	AudioPlay *play = data;
+	gchar *message;
+
+	g_assert (IS_AUDIO_PLAY (play));
+
+	message = g_strdup_printf ("GStreamer error: %s\n", error);
+	audio_play_error (play, message);
+	g_free (message);
+}
+
 /* callback used by query of length of track to know when caps are there */
 static void
 deep_notify (GObject *object, GstObject *origin,
 	     GParamSpec *pspec, AudioPlay *play)
 {
 	GValue value = { 0 };
+	gint rate = 0;
+
 	/* don't care about file offsets */
 	if (strcmp (pspec->name, "offset") == 0) return;
 	if (strcmp (pspec->name, "caps") == 0)
 	{
-		g_print ("DEBUG: got caps\n");
+		NM_DEBUG("got caps\n");
+		/* FIXME: we should check if the caps are sufficient
+		 * for length calculation */
 		g_value_init (&value, pspec->value_type);
 		g_object_get_property (G_OBJECT (origin), pspec->name, &value);
 		play->priv->caps = g_value_peek_pointer (&value);
-		play->priv->have_caps = TRUE;
+		if (gst_caps_get_props (play->priv->caps) == NULL)
+		{
+			NM_DEBUG("no caps yet, returning\n");
+		}
+		else
+			play->priv->have_caps = TRUE;
 	}
 	if (strcmp (pspec->name, "channels") == 0)
 	{
 		/* good enough to have_caps on */
 		play->priv->have_caps = TRUE;
 	}
-	g_print ("DEBUG: deep_notify: %s\n", pspec->name);
+	NM_DEBUG("deep_notify: %s\n", pspec->name);
 }
-/* callback for when we have the type of the file 
+/* callback for when we have the type of the file
+ * set the boolean and handle it in the iterator
  * we set up a decoder based on the mime type
  */
-static void 
-have_type (GstElement *element, GstCaps *caps, AudioPlay *play) 
+static void
+have_type (GstElement *element, GstCaps *caps, AudioPlay *play)
 {
 	AudioPlayPriv *priv = play->priv;
-  	gchar *mime = g_strdup_printf (gst_caps_get_mime (caps));
-	GstElement *decoder = NULL;
 
 #ifdef DEBUG
-	g_print ("DEBUG: have_type callback called.\n");
-  	fprintf (stderr, "have caps, mime type is %s\n", mime);
+	NM_DEBUG("have_type callback called.\n");
 #endif
 
 	g_assert (GST_IS_ELEMENT (priv->pipeline));
-  	gst_element_set_state (priv->pipeline, GST_STATE_READY);
-
-  	/* disconnect the typefind from the pipeline and remove it,
-   	 * since we now know what the type is */
-	gst_element_disconnect (priv->src, priv->typefind);
-	/* ref it so it doesn't go away */
-	g_object_ref (priv->typefind);
-  	gst_bin_remove (GST_BIN (priv->pipeline), priv->typefind);
-
-  	/* gst_scheduler_show (GST_ELEMENT_SCHED (priv->pipeline)); */
-
-  	/* now based on the mime type set up the pipeline properly */
-  	if (strstr (mime, "mp3")) 
-    		decoder = gst_element_factory_make ("mad", "decoder");
-  	else if (strstr (mime, "x-ogg"))
-    		decoder = gst_element_factory_make ("vorbisfile", "decoder");
-  	else if (strstr (mime, "x-wav"))
-    		decoder = gst_element_factory_make ("wavparse", "decoder");
-  	else if (strstr (mime, "x-flac"))
-    		decoder = gst_element_factory_make ("flacdec", "decoder");
-  	else 
-	{
-    		g_print ("mime type %s not handled in this program.\n", mime);
-    		g_free (mime);
-		return;
-	}
-  	g_free (mime);
-	priv->decoder = decoder;
-
-	/* set it to null */
-  	gst_element_set_state (priv->pipeline, GST_STATE_NULL);
-
-  	/* now put playback pipe in place of where typefind was  */
-	g_print ("DEBUG: putting in decoder\n");
-  	gst_bin_add (GST_BIN (priv->pipeline), priv->decoder);
-	g_print ("DEBUG: putting in sink\n");
-  	gst_bin_add (GST_BIN (priv->pipeline), priv->sink);
-	/* FIXME: this shouldn't go here */
-	g_signal_connect (G_OBJECT (play->priv->sink), "eos",
-			  G_CALLBACK (callback_sink_eos), play);
-	if (!gst_element_connect (priv->src, priv->decoder))
-		g_error ("warning: could not connect src and decoder !\n");
-	if (!gst_element_connect (priv->decoder, priv->sink))
-		g_print ("warning: could not connect decoder and sink !\n");
-  	if (gst_element_set_state (priv->pipeline, GST_STATE_PAUSED)
-			== GST_STATE_FAILURE)
-		g_print ("pipelien could not be paused !\n");
-  	/* gst_scheduler_show (GST_ELEMENT_SCHED (priv->pipeline)); */
+	priv->have_type = TRUE;
+	priv->caps = caps;
 }
+
 /* object functions */
 
 static void
@@ -308,11 +380,12 @@ audio_play_class_init (AudioPlayClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_ref (GST_TYPE_OBJECT);
+        parent_class = g_type_class_ref (G_TYPE_OBJECT);
 
         klass->eos = NULL;
 
-        //gobject_class->dispose = audio_play_dispose;
+        gobject_class->dispose = audio_play_dispose;
+        gobject_class->finalize = audio_play_finalize;
 
         audio_play_signals [EOS_SIGNAL] =
 		g_signal_new ("eos",
@@ -373,10 +446,10 @@ audio_play_init (AudioPlay *play)
  * and pre-initializes typefind
  */
 
-AudioPlay * 
+AudioPlay *
 audio_play_new (GError **error)
 {
-	AudioPlay *play = g_object_new (AUDIO_TYPE_PLAY, NULL);
+	AudioPlay *play = g_object_new (AUDIO_PLAY_TYPE, NULL);
 	AudioPlayPriv *priv = play->priv;
 
 	priv->pipeline = gst_pipeline_new ("pipeline");
@@ -385,11 +458,29 @@ audio_play_new (GError **error)
 	/* set up all the elements we're going to use */
 
 	priv->src = gst_element_factory_make ("gnomevfssrc", "src");
-	g_assert (priv->src);
+	if (priv->src == NULL)
+	{
+		if (!error) return NULL;
+		/* FIXME: I18N !!! */
+		*error = g_error_new (AUDIO_PLAY_ERROR,
+				      AUDIO_PLAY_ERROR_SRC,
+				     "GStreamer: Could not create %s element",
+				     "gnomevfssrc");
+		return NULL;
+	}
 
 	priv->typefind = gst_element_factory_make ("typefind", "typefind");
-	g_assert (priv->typefind);
-
+	if (priv->typefind == NULL)
+	{
+		gst_object_unref (GST_OBJECT (priv->src));
+		priv->src = NULL;
+		if (!error) return NULL;
+		*error = g_error_new (AUDIO_PLAY_ERROR,
+				      AUDIO_PLAY_ERROR_TYPEFIND,
+				     "GStreamer: Could not create %s element",
+				     "typefind");
+		return NULL;
+	}
 	gst_bin_add (GST_BIN (priv->pipeline), priv->src);
 
 	/* execute the callback when we find the type */
@@ -399,42 +490,110 @@ audio_play_new (GError **error)
 	/* catch deep notifies */
 	g_signal_connect (priv->pipeline, "deep_notify",
 		          G_CALLBACK (deep_notify), play);
+
+	/* catch errors, they're fun ! */
+	g_signal_connect (priv->pipeline, "error",
+			  G_CALLBACK (error_callback), play);
+
 	priv->have_type = FALSE;
 	priv->have_eos = FALSE;
+	priv->error_handler = NULL;
+	priv->error_data = NULL;
 	play->priv = priv;
 
-	g_print ("DEBUG: audio_play_new done\n");
+	NM_DEBUG("audio_play_new done\n");
 	audio_play_debug (play);
 	return play;
 }
 
-/* free all resources held by the play object */
 void
-audio_play_dispose (AudioPlay *play)
+audio_play_dispose (GObject *obj)
 {
+	AudioPlay *play;
 	AudioPlayPriv *priv;
-       
-	g_print ("DEBUG: audio_play_dispose\n");
+
+	play = AUDIO_PLAY (obj);
+
+	NM_DEBUG("audio_play_dispose\n");
 	g_assert (play != NULL);
 	g_return_if_fail (IS_AUDIO_PLAY (play));
 	if (gst_element_get_state (play->priv->pipeline) == GST_STATE_PLAYING)
-		audio_play_playing_to_ready (play);	
+		audio_play_playing_to_ready (play);
 
-	g_print ("DEBUG: dispose: getting priv\n");
+	NM_DEBUG("dispose: getting priv\n");
 	priv = play->priv;
 
+	NM_DEBUG("dispose: setting pipeline to NULL\n");
 	gst_element_set_state (priv->pipeline, GST_STATE_NULL);
-	g_print ("DEBUG: dispose: set to null\n");
-	/*
+
+	if (priv->tick_timeout_id) {
+		g_source_remove (priv->tick_timeout_id);
+		priv->tick_timeout_id = 0;
+	}
+
+	if (priv->iterator_id != 0)
+	{
+		g_source_remove (priv->iterator_id);
+		priv->iterator_id = 0;
+	}
+	if (priv->idler_id != 0)
+	{
+		g_source_remove (priv->idler_id);
+		priv->idler_id = 0;
+	}
+	NM_DEBUG("dispose: removing decoder\n");
 	if (priv->decoder)
 	{
 		gst_bin_remove (GST_BIN (priv->pipeline), priv->decoder);
+		g_object_unref (priv->decoder);
+		priv->decoder = NULL;
+	}
+
+	NM_DEBUG("dispose: unreffing typefind\n");
+	if (priv->typefind)
+	{
+		g_object_unref (priv->typefind);
+		priv->typefind = NULL;
+	}
+
+	NM_DEBUG("dispose: unreffing src\n");
+	if (priv->src)
+	{
+		g_object_unref (priv->src);
+		priv->src = NULL;
+	}
+
+	NM_DEBUG("dispose: unreffing sink\n");
+	if (priv->sink)
+	{
+		g_object_unref (priv->sink);
+		priv->sink = NULL;
+	}
+
+	/* FIXME: for some reason this one is complaining heavily
+	NM_DEBUG("dispose: unreffing pipeline\n");
+	if (priv->pipeline)
+	{
+		g_object_unref (priv->pipeline);
+		priv->pipeline = NULL;
 	}
-	g_object_unref (priv->typefind);
-	g_object_unref (priv->src);
-	g_object_unref (priv->sink);
-	g_object_unref (priv->pipeline);
 	*/
+	NM_DEBUG("dispose: done unreffing\n");
+}
+
+/* Free all the resources held by the object */
+void
+audio_play_finalize (GObject *obj)
+{
+	AudioPlay *play;
+	AudioPlayPriv *priv;
+
+	play = AUDIO_PLAY (obj);
+
+	priv = play->priv;
+
+	g_free (priv->location);
+	g_free (priv);
 }
 
 /* prerequisites:
@@ -446,7 +605,7 @@ audio_play_autoplug (AudioPlay *play, GError **error)
 {
 	AudioPlayPriv *priv = play->priv;
 #ifdef DEBUG
-	g_print ("DEBUG: starting autoplug\n");
+	NM_DEBUG("starting autoplug\n");
 #endif
 	g_assert (GST_IS_BIN (priv->pipeline));
 
@@ -466,9 +625,9 @@ audio_play_autoplug (AudioPlay *play, GError **error)
 	gst_element_set_state ((play->priv->pipeline), GST_STATE_PLAYING);
 	/* gst_scheduler_show (GST_ELEMENT_SCHED (play->priv->pipeline)); */
 	audio_play_debug (play);
-	g_print ("DEBUG: starting iterating for autoplug\n");
+	NM_DEBUG("starting iterating for autoplug\n");
 	g_assert (gst_element_get_state (play->priv->pipeline) == GST_STATE_PLAYING);
-	while (play->priv->decoder == NULL)
+	while (play->priv->have_type == FALSE)
 	{
 		g_print ("*");
 	       if (!gst_bin_iterate (GST_BIN (play->priv->pipeline)))
@@ -486,9 +645,10 @@ audio_play_autoplug (AudioPlay *play, GError **error)
 		       return FALSE;
 	       }
 	}
-	/* we have the type */
+	/* we have the type, handle it */
+	handle_have_type (play);
 #ifdef DEBUG
-	g_print ("DEBUG: done autoplug\n");
+	NM_DEBUG("done autoplug\n");
 #endif
 	return TRUE;
 }
@@ -498,7 +658,7 @@ gboolean
 audio_play_set_location (AudioPlay *play, const char *uri, GError **error)
 {
 #ifdef DEBUG
-	g_print ("DEBUG: loading location %s\n", uri);
+	NM_DEBUG("loading location %s\n", uri);
 #endif
 	if (gst_element_get_state (play->priv->pipeline) == GST_STATE_PLAYING)
 	{
@@ -510,7 +670,7 @@ audio_play_set_location (AudioPlay *play, const char *uri, GError **error)
 	g_object_set (G_OBJECT (play->priv->src), "location", uri, NULL);
 
 	/* clear values that need clearing */
-	play->priv->seconds = 0;
+	play->priv->seconds = -1; /* this ensures we get a tick at 0 too */
 	play->priv->nanoseconds = 0;
 	play->priv->length_nanoseconds = 0;
 	play->priv->have_caps = FALSE;
@@ -548,7 +708,7 @@ audio_play_update_length (AudioPlay *play)
 	if (caps == NULL) g_print ("WARNING: caps are NULL\n");
 	gst_caps_debug (caps, "caps of decoder src pad");
 
-	res = gst_pad_query (pad, GST_PAD_QUERY_TOTAL, &format, &value);
+	res = gst_pad_query (pad, GST_QUERY_TOTAL, &format, &value);
 	if (!res) 
 	{
 		g_print ("WARNING: pad query failed,can't get length !\n");
@@ -557,7 +717,7 @@ audio_play_update_length (AudioPlay *play)
 	}
 	g_assert (res == TRUE);
 	g_assert (format == GST_FORMAT_TIME);
-	g_print ("DEBUG: length in %s: %lld (nanosecs)\n", definition->nick, 
+	NM_DEBUG("length in %s: %lld (nanosecs)\n", definition->nick, 
 		 value);
 	play->priv->length_nanoseconds = value;
 	g_signal_emit (G_OBJECT (play), 
@@ -576,7 +736,7 @@ audio_play_seek_to_time (AudioPlay *play, guint64 nanosecs)
 		seek_time = 0LL;
 	else
 		seek_time = (gint64) nanosecs;
-	g_print ("DEBUG: audio_play_seek_to_time: seeking to %lld\n", seek_time);
+	NM_DEBUG("audio_play_seek_to_time: seeking to %lld\n", seek_time);
 	gst_element_set_state (play->priv->pipeline, GST_STATE_PAUSED);
 	event = gst_event_new_seek (GST_FORMAT_TIME |
 			            GST_SEEK_METHOD_SET |
@@ -586,10 +746,14 @@ audio_play_seek_to_time (AudioPlay *play, guint64 nanosecs)
 	if (gst_element_send_event (play->priv->sink, event))
 	{
 		play->priv->nanoseconds = nanosecs;
-		g_signal_emit (G_OBJECT (play), 
+		g_signal_emit (G_OBJECT (play),
 			       audio_play_signals [TICK_SIGNAL], 0,
 			       play->priv->nanoseconds);
 	}
+	else
+	{
+		g_error ("DEBUG: gst_element_send_event: FAILED\n");
+	}
 	gst_element_set_state (play->priv->pipeline, GST_STATE_PLAYING);
 }
 
@@ -598,57 +762,86 @@ void
 audio_play_seek_to_pos (AudioPlay *play, double value)
 {
 	guint64 seek_to = (guint64) ((double) play->priv->length_nanoseconds * value);
-	g_print ("DEBUG: audio_play_seek_to_pos: seeking to %lld\n", seek_to);
-	g_print ("DEBUG: value was %f\n", value);
+	NM_DEBUG("audio_play_seek_to_pos: seeking to %lld\n", seek_to);
+	NM_DEBUG("value was %f\n", value);
 	audio_play_seek_to_time (play, seek_to);
 }
 
+/* state change functions */
+/* splitting these specific ones up is a good idea because it's cleaner
+ * and less indented */
+
+static gboolean
+audio_play_set_state_playing (AudioPlay *play, GstElementState state,
+		              GError **error)
+{
+	NM_DEBUG("playing audio play\n");
+	/* FIXME: somewhere else ? update clock */
+	play->priv->clock = gst_bin_get_clock (GST_BIN (play->priv->pipeline));
+	if (play->priv->decoder == NULL)
+	{
+		audio_play_error (play, "no decoder found yet");
+		return FALSE;
+	}
+	if (gst_element_set_state (play->priv->pipeline, GST_STATE_PLAYING) ==
+				   GST_STATE_FAILURE)
+	{
+		/*
+		 * FIXME: this error is due to a core error, so somehow
+		 * find out if the core already signaled an error and if it
+		 * did,  we don't need to show this one */
+		/*
+		audio_play_error (play, "could not set play pipeline");
+		*/
+		return FALSE;
+	}
+	audio_play_update_length (play);
+
+	/* add idlers */
+	/* handle signals asyncly */
+	play->priv->idler_id =
+		g_idle_add ((GSourceFunc) idler, play);
+	g_assert (play->priv->idler_id > 0);
+	/* set handler for playback; it will unregister
+	 * when iterate fails */
+	play->priv->iterator_id =
+		g_idle_add_full (G_PRIORITY_DEFAULT_IDLE - 10,
+				 (GSourceFunc) iterator,
+				 play, NULL);
+	g_assert (play->priv->iterator_id > 0);
+	/* set tick timeout */
+	play->priv->tick_timeout_id = g_timeout_add (200, (GSourceFunc) tick_timeout, play);
+}
 gboolean
 audio_play_set_state (AudioPlay *play, GstElementState state, GError **error)
 {
 	switch (state)
 	{
 		case GST_STATE_PLAYING:
-			g_print ("DEBUG: playing audio play\n");
-			/* FIXME: somewhere else ? update clock */
-			play->priv->clock = 
-				gst_bin_get_clock (GST_BIN (play->priv->pipeline));
-			if (play->priv->decoder == NULL)
-			{
-				g_print ("DEBUG: can't play, no decoder yet\n");
-				return FALSE;
-			}
-			gst_element_set_state (play->priv->pipeline, 
-					       GST_STATE_PLAYING);
-			audio_play_update_length (play);
-
-			/* add idlers */
-			/* handle signals asyncly */
-			play->priv->idler_id = 
-				g_idle_add ((GSourceFunc) idler, play);
-			g_assert (play->priv->idler_id > 0);
-			/* set handler for playback; it will unregister
-			 * when iterate fails */
-			play->priv->iterator_id = 
-				g_idle_add ((GSourceFunc) iterator, 
-				            play);
-			g_assert (play->priv->iterator_id > 0);
-			/* set tick timeout */
-			g_timeout_add (200, (GSourceFunc) tick_timeout, play);
+			return audio_play_set_state_playing (play, state, error);
 			break;
 		case GST_STATE_PAUSED:
-			g_print ("DEBUG: pausing audio play\n");
-			gst_element_set_state (play->priv->pipeline,
-					       GST_STATE_PAUSED);
+			NM_DEBUG("pausing audio play\n");
+			if (gst_element_set_state (play->priv->pipeline,
+					           GST_STATE_PAUSED) ==
+					           GST_STATE_FAILURE)
+			{
+				audio_play_error (play, "Could not move to "
+						        "'paused' state.");
+				return FALSE;
+			};
 			break;
 		case GST_STATE_READY:
-			g_print ("DEBUG: going to ready");
+			NM_DEBUG("going to ready");
 			if (play->priv->decoder)
 			{
 				if (!audio_play_playing_to_ready (play))
-					g_print ("couldn't play->ready\n");
+				{
+					audio_play_error (play,
+							  "couldn't go from playing to ready");
+					return FALSE;
+				}
 			}
-			g_print ("DEBUG: decoder: %p\n", play->priv->decoder);
 			break;
 		default:
 			g_print ("unhandled state\n");
@@ -665,8 +858,6 @@ audio_play_get_state (AudioPlay *play)
 guint64
 audio_play_get_length (AudioPlay *play)
 {
-	g_print ("DEBUG: audio_play_get_length: returning %lld\n",
-		 play->priv->length_nanoseconds);
 	return play->priv->length_nanoseconds;
 }
 
@@ -684,6 +875,15 @@ audio_play_set_audio_sink (AudioPlay *play, GstElement *element)
 	g_signal_connect (G_OBJECT (play->priv->sink), "eos",
 			  G_CALLBACK (callback_sink_eos), play);
 }
+/* set an error handler on the play object to call on errors. */
+void
+audio_play_set_error_handler (AudioPlay *play,
+		              AudioPlayErrorHandler handler, gpointer data)
+{
+	g_assert (IS_AUDIO_PLAY (play));
+	play->priv->error_handler = handler;
+	play->priv->error_data = data;
+}
 
 /* move from playing to ready */
 static gboolean
@@ -692,7 +892,7 @@ audio_play_playing_to_ready (AudioPlay *play)
 	AudioPlayPriv *priv = play->priv;
 	/* stop the pipepline */
 	gst_element_set_state (priv->pipeline, GST_STATE_READY);
-	
+
 	/* disconnect src ! decoder ! sink */
 	gst_element_disconnect (priv->src, priv->decoder);
 	gst_element_disconnect (priv->decoder, priv->sink);
@@ -719,10 +919,10 @@ audio_play_playing_to_ready (AudioPlay *play)
 
 	/* reconnect typefind to source and readd to bin */
 	/*
-	g_print ("DEBUG: reconnecting typefind\n");
+	NM_DEBUG("reconnecting typefind\n");
 	gst_element_connect (priv->src, priv->typefind);
 	gst_bin_add (GST_BIN (priv->pipeline), priv->typefind);
 	*/
-	g_print ("DEBUG: play to ready done\n");
+	NM_DEBUG("play to ready done\n");
 	return TRUE;
 }
