@@ -103,6 +103,7 @@ struct AudioView {
 	gchar *error_message;
 
 	/* state information for idler */
+	guint media_info_idler_id;
 	AudioViewInfoState state;
 	GstMediaInfoStream *stream;	/* set by idler read */
 	GList *audio_item;	/* pointer to current audio_list item under
@@ -148,14 +149,22 @@ audio_view_tree_model_dump (AudioView *view)
 /* helper functions */
 static GtkWidget *
 control_box_add_labeled_button (GtkWidget *control,
-		                const gchar *label, const gchar *file)
+		                const gchar *label, const gchar *file,
+				gboolean toggle)
 {
 	GtkWidget *button;
 	GdkPixbuf *pixbuf;
 	GtkWidget *image;
 	gchar *location;
 
-	button = gtk_button_new ();
+	if (toggle)
+	{
+		button = gtk_toggle_button_new ();
+	}
+	else
+	{
+		button = gtk_button_new ();
+	}
 
 	location = g_strdup_printf ("%s/%s", PIXMAPSDIR, file);
 	pixbuf = gdk_pixbuf_new_from_file (location, NULL);
@@ -187,8 +196,12 @@ audio_view_is_playable (const char *uri)
             (strcmp (type, "application/x-ogg") != 0) &&
             (strcmp (type, "audio/x-flac") != 0) &&
             (strcmp (type, "audio/x-mp3") != 0) &&
-            (strcmp (type, "audio/x-wav") != 0)
-            )
+            (strcmp (type, "audio/x-wav") != 0) &&
+	    (strcmp (type, "audio/x-mod") != 0) &&
+	    (strcmp (type, "audio/x-s3m") != 0) &&
+	    (strcmp (type, "audio/x-xm") != 0) &&
+	    (strcmp (type, "audio/x-it") != 0)
+	   )
         {
                 g_free (type);
                 return NULL;
@@ -231,6 +244,37 @@ audio_view_props_to_string (GList *list, const gchar *prepend)
 	}
 	return result;
 }
+/* returns a newly allocated human-readable type for the given mime type */
+static gchar *
+audio_view_mime_to_type (const gchar *mime)
+{
+	gchar *result;
+        if (strcmp (mime, "application/x-ogg") == 0)
+		return g_strdup (_("Ogg/Vorbis"));
+        if (strcmp (mime, "application/x-flac") == 0)
+		return g_strdup (_("FLAC"));
+        if (strcmp (mime, "audio/x-mp3") == 0)
+		return g_strdup (_("MPEG"));
+        if (strcmp (mime, "audio/x-wav") == 0)
+		return g_strdup (_("WAVE"));
+	/* FIXME: add more mimes prematurely so they are translated when
+	 * we need them ! */
+        if (strcmp (mime, "audio/x-mod") == 0)
+		return g_strdup (_("Amiga mod"));
+	if (strcmp (mime, "audio/x-s3m") == 0)
+		return g_strdup (_("Amiga mod"));
+	if (strcmp (mime, "audio/x-xm") == 0)
+		return g_strdup (_("Amiga mod"));
+	if (strcmp (mime, "audio/x-it") == 0)
+		return g_strdup (_("Amiga mod"));
+        if (strcmp (mime, "audio/x-aiff") == 0)
+		return g_strdup (_("Apple AIFF"));
+        if (strcmp (mime, "audio/x-midi") == 0)
+		return g_strdup (_("MIDI"));
+        if (strcmp (mime, "audio/x-ulaw") == 0)
+		return g_strdup (_("ulaw audio"));
+	return g_strdup (_("Unknown"));
+}
 
 /* extract the useful metadata to an allocated string */
 static gchar *
@@ -245,7 +289,7 @@ audio_view_media_get_metadata (GstMediaInfoStream *info)
 	GList *p;
 	GstMediaInfoTrack *track;
 
-	g_print ("DEBUG: getting metadata\n");
+	NM_DEBUG("getting metadata\n");
 	if (info->length_tracks < 1) return NULL;
 	if (info->tracks == NULL) return NULL;
 
@@ -276,8 +320,8 @@ audio_view_media_get_metadata (GstMediaInfoStream *info)
 	}
 
 	/* now construct a string out of it */
-	artists = audio_view_props_to_string (artist, "Artist: ");
-	titles = audio_view_props_to_string (title, "Title: ");
+	artists = audio_view_props_to_string (artist, _("Artist: "));
+	titles = audio_view_props_to_string (title, _("Title: "));
 	if (artists && titles)
 	{
 		metadata = g_strconcat (artists, titles, NULL);
@@ -304,7 +348,7 @@ audio_view_play_next (AudioView *view)
 	GstMediaInfoStream *info;
 
 	g_assert (IS_AUDIO_PLAY (play));
-	g_print ("DEBUG: play_next: start\n");
+	NM_DEBUG("play_next: start\n");
 
 	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->tree_view));
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->tree_view));
@@ -318,15 +362,15 @@ audio_view_play_next (AudioView *view)
 		gtk_tree_path_next (path);
 		if (!gtk_tree_model_get_iter (model, &iter, path))
 		{
-			g_print ("DEBUG: play_next: already last\n");
-			//gtk_tree_selection_unselect_path (sel, old_path);
+			NM_DEBUG("play_next: already last\n");
+			/* gtk_tree_selection_unselect_path (sel, old_path); */
 			gtk_tree_path_free (path);
 			return FALSE;
 		}
 		gtk_tree_model_get (model, &iter,
 				    AUDIO_INFO_COLUMN, &info,
 				    -1);
-		g_print ("DEBUG: play_next: track to play is %s\n", info->path);
+		NM_DEBUG("play_next: track to play is %s\n", info->path);
 	}
 	audio_view_set_playing (view, info, path);
 	gtk_tree_path_free (path);
@@ -347,7 +391,7 @@ audio_view_set_time (AudioView *view, gint seconds, gint length)
 {
 	gchar *text;
 	if (length == 0)
-		text = g_strdup_printf ("<span size=\"larger\">Unknown</span>");
+		text = g_strdup_printf (_("<span size=\"larger\">Unknown</span>"));
 	else
 		text = g_strdup_printf ("<span size=\"larger\">"
 				        "[%d:%02d/%d.%02d]</span>", 
@@ -382,10 +426,14 @@ audio_view_set_playing (AudioView *view, GstMediaInfoStream *info,
 	gchar *unescaped;
 	AudioPlay *play;
 	GError **error = NULL;
+	GtkTreeModel *model;
+	GtkTreeSelection *sel;
+	GtkTreeIter iter;
+	gchar *filename;
 
 	if (info == NULL)
 	{
-		message = g_strdup_printf ("ERROR: no information yet");
+		message = g_strdup_printf (_("ERROR: no information yet"));
 		gtk_label_set_text (GTK_LABEL (view->status), message);
 		g_free (message);
 		return;
@@ -395,11 +443,11 @@ audio_view_set_playing (AudioView *view, GstMediaInfoStream *info,
 	g_assert (IS_AUDIO_PLAY (play));
 	if (audio_play_get_state (play) == GST_STATE_PLAYING)
 	{
-		g_print ("DEBUG: set_playing: was playing, set to READY\n");
+		NM_DEBUG("set_playing: was playing, set to READY\n");
 		audio_play_set_state (play, GST_STATE_READY, NULL);
-		g_print ("DEBUG: set_playing: changed state to READY\n");
+		NM_DEBUG("set_playing: changed state to READY\n");
 	}
-	g_print ("DEBUG: set_playing: setting %s to play\n", info->path);
+	NM_DEBUG("set_playing: setting %s to play\n", info->path);
 	audio_play_set_location (play, info->path, NULL);
 	audio_play_set_state (play, GST_STATE_PLAYING, error);
 	if (view->scanning)
@@ -411,13 +459,13 @@ audio_view_set_playing (AudioView *view, GstMediaInfoStream *info,
 			g_source_remove (view->scan_timeout);
 			view->scan_timeout = -1;
 		}
-		g_print ("DEBUG: we're scanning, %lld\n", nanosecs);
+		NM_DEBUG("we're scanning, %lld\n", nanosecs);
 		if (nanosecs > SCAN_LENGTH * GST_SECOND)
 		{
 			nanosecs -= SCAN_LENGTH;
 			nanosecs /= 2;
-			g_print ("DEBUG: scanning mode, seek to somewhere in middle\n");
-			g_print ("DEBUG: seeking to %lld\n", nanosecs);
+			NM_DEBUG("scanning mode, seek to somewhere in middle\n");
+			NM_DEBUG("seeking to %lld\n", nanosecs);
 			audio_play_seek_to_time (play, nanosecs);
 			view->scan_timeout = g_timeout_add (SCAN_LENGTH * 1000,
 					                    (GSourceFunc) scan_timeout,
@@ -427,7 +475,7 @@ audio_view_set_playing (AudioView *view, GstMediaInfoStream *info,
 	if (error)
 	{
 		/* FIXME: handleme */
-		message = g_strdup_printf ("ERROR: %s", (*error)->message);
+		message = g_strdup_printf (_("ERROR: %s"), (*error)->message);
 		gtk_label_set_text (GTK_LABEL (view->status), message);
 		g_free (message);
 		g_free (error);
@@ -435,14 +483,36 @@ audio_view_set_playing (AudioView *view, GstMediaInfoStream *info,
 	}
 
 	/* status update */
-	message = g_strdup_printf ("Playing %s", info->path);
+	message = g_strdup_printf (_("Playing %s"), info->path);
 	gtk_label_set_text (GTK_LABEL (view->status), message);
 	g_free (message);
 
+	NM_DEBUG("updating selection\n");
 	/* selection update */
 	if (path)
+	{
+
 		gtk_tree_view_set_cursor (GTK_TREE_VIEW (view->tree_view),
 			                  path, NULL, FALSE);
+	}
+
+	/* now playing update */
+	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->tree_view));
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->tree_view));
+	g_assert (GTK_IS_TREE_MODEL (model));
+
+	if (gtk_tree_selection_get_selected (sel, &model, &iter))
+	{
+		gtk_tree_model_get (model, &iter, FILE_COLUMN, &filename, -1);
+		NM_DEBUG("filename is %s\n", filename);
+		message = g_strdup_printf (_("Playing %s"), filename);
+		gtk_label_set_text (GTK_LABEL (view->status), message);
+		g_free (message);
+		gtk_tree_model_get (model, &iter,
+				    AUDIO_INFO_COLUMN, &info,
+				    -1);
+	}
+	return;
 }
 
 /* ui callbacks */
@@ -481,7 +551,7 @@ row_activated_callback (GtkTreeView *tree_view, GtkTreePath *path,
 	GtkTreeModel *model = gtk_tree_view_get_model (tree_view);
 	GstMediaInfoStream *info;
 	gchar *pathname = gtk_tree_path_to_string (path);
-	g_print ("DEBUG: activated_cb: path %s\n", pathname);
+	NM_DEBUG("activated_cb: path %s\n", pathname);
 	g_free (pathname);
 
 	if (! (gtk_tree_model_get_iter (model, &iter, path)))
@@ -490,7 +560,7 @@ row_activated_callback (GtkTreeView *tree_view, GtkTreePath *path,
 	gtk_tree_model_get (model, &iter,
 			    AUDIO_INFO_COLUMN, &info,
 			    -1);
-	g_print ("DEBUG: callback: get info %p (%s) playing (iterp %)\n",
+	NM_DEBUG("callback: get info %p (%s) playing (iterp %)\n",
 		 info, info->path, &iter);
 	audio_view_set_playing (view, info, NULL);
 }
@@ -547,12 +617,12 @@ play_activate (GtkButton *play_button, AudioView *view)
 		gtk_tree_model_get (model, &iter,
 				    AUDIO_INFO_COLUMN, &info,
 				    -1);
-		g_print ("DEBUG: pressed play on track %s\n", info->path);
+		NM_DEBUG("pressed play on track %s\n", info->path);
 		audio_view_set_playing (view, info, NULL);
 	}
 	else
 	{
-		message = g_strdup_printf ("Make a selection first !");
+		message = g_strdup_printf (_("Make a selection first !"));
 		gtk_label_set_text (GTK_LABEL (view->status), message);
 		g_free (message);
 	}
@@ -567,7 +637,7 @@ stop_activate (GtkButton *stop_button, AudioView *view)
 	g_assert (IS_AUDIO_PLAY (play));
 	if (audio_play_get_state (play) == GST_STATE_PLAYING)
 	{
-		g_print ("DEBUG: pressed stop, setting to ready\n");
+		NM_DEBUG("pressed stop, setting to ready\n");
 		audio_play_set_state (play, GST_STATE_READY, NULL);
 	}
 	/* remove callback if there */
@@ -580,7 +650,7 @@ stop_activate (GtkButton *stop_button, AudioView *view)
 			     (double) 0.0);
 	audio_view_set_time (view, 0, 0);
 	gtk_label_set_text (GTK_LABEL (view->status),
-			    "Stopped.");
+			    _("Stopped."));
 }
 
 static void
@@ -592,9 +662,9 @@ next_activate (GtkButton *next_button, AudioView *view)
 static void
 scan_activate (GtkButton *scan_button, AudioView *view)
 {
-	g_print ("DEBUG: toggle scan\n");
+	NM_DEBUG("toggle scan\n");
 	view->scanning = !view->scanning;
-	g_print ("DEBUG: scanning %s\n", view->scanning ? "true" : "false");
+	NM_DEBUG("scanning %s\n", view->scanning ? "true" : "false");
 	/* FIXME: update toggle ? */
 	/* FIXME: act when playing or smth ? */
 	if (view->scanning)
@@ -625,7 +695,7 @@ have_tick_callback (AudioPlay *play, gint64 time_nanos, AudioView *view)
 	gint seconds = (gint) (time_nanos / GST_SECOND);
 
 	length_seconds = (gint) (audio_play_get_length (play) / GST_SECOND);
-	g_print ("DEBUG: tick: length %d\n", length_seconds);
+	NM_DEBUG("tick: length %d\n", length_seconds);
 	/* update time display */
 	audio_view_set_time (view, seconds, length_seconds);
 
@@ -644,11 +714,11 @@ have_tick_callback (AudioPlay *play, gint64 time_nanos, AudioView *view)
 static void
 have_eos_callback (AudioPlay *play, AudioView *view)
 {
-	g_print ("DEBUG: have_eos callback activated\n");
-	//audio_play_set_state (play, GST_STATE_READY, NULL);
+	NM_DEBUG("have_eos callback activated\n");
+	/* audio_play_set_state (play, GST_STATE_READY, NULL); */
 	if (! audio_view_play_next (view))
 	{
-		gtk_label_set_text (GTK_LABEL (view->status), "Stopped.");
+		gtk_label_set_text (GTK_LABEL (view->status), _("Stopped."));
 		gtk_widget_show (view->status);
 	}
 }
@@ -693,11 +763,16 @@ audio_view_get_media_info_idler (AudioView *view)
 	GstMediaInfoStream *info = NULL;
 	gchar *metadata;
 
+	if (view->audio_list == NULL)
+	{
+		/* no audio items at all ! */
+		return FALSE;
+	}
 	switch (view->state)
 	{
 		case AUDIO_VIEW_INFO_STATE_NULL:
 		{
-			g_print ("DEBUG: STATE is NULL\n");
+			NM_DEBUG("STATE is NULL\n");
 			view->audio_item = view->audio_list;
 			view->state = AUDIO_VIEW_INFO_STATE_START;
 			return TRUE;
@@ -711,14 +786,14 @@ audio_view_get_media_info_idler (AudioView *view)
 			view->iterp = (GtkTreeIter *) view->audio_item->data;
 			gtk_tree_model_get (GTK_TREE_MODEL (view->list_store),
 					    view->iterp, FILE_COLUMN, &uri, -1);
-			g_print ("DEBUG: get: view->iterp: %p (%s)\n",
+			NM_DEBUG("get: view->iterp: %p (%s)\n",
 				 view->iterp, uri);
 
 			escaped_name = (char *) gnome_vfs_escape_string (uri);
 			path_uri = g_build_filename (view->location,
 						     uri, NULL);
 			g_free (escaped_name);
-			g_print ("getting media-info for path uri %s\n",
+			NM_DEBUG("getting media-info for path uri %s\n",
 			         path_uri);
 			gst_media_info_read_with_idler (view->media_info,
 							path_uri,
@@ -733,6 +808,7 @@ audio_view_get_media_info_idler (AudioView *view)
 			gchar *time;
 			gchar *bitrate;
 			gchar *metadata;
+			gchar *type;
 
 			if ((view->stream) == NULL)
 			{
@@ -759,23 +835,25 @@ audio_view_get_media_info_idler (AudioView *view)
 					bitrate = g_strdup_printf ("%d kbs",
 						  info->bitrate / 1000);
 				else
-					bitrate = g_strdup_printf ("Unknown");
+					bitrate = g_strdup_printf (_("Unknown"));
 			}
 			else
 			{
-				time = g_strdup ("Unkown");
-				metadata = g_strdup ("Unknown");
+				time = g_strdup (_("Unknown"));
+				metadata = g_strdup (_("Unknown"));
 			}
-			g_print ("DEBUG: set: view->iterp: %p (%s)\n",
+			NM_DEBUG("set: view->iterp: %p (%s)\n",
 				 view->iterp, info->path);
+			type = audio_view_mime_to_type (info->mime);
 			gtk_list_store_set (view->list_store,
 					    view->iterp,
 					    LENGTH_COLUMN, time,
 					    BITRATE_COLUMN, bitrate,
 					    METADATA_COLUMN, metadata,
-					    TYPE_COLUMN, info->mime,
+					    TYPE_COLUMN, type,
 					    AUDIO_INFO_COLUMN, info,
 					    -1);
+			/* FIXME: do we have to free type ? */
 			view->stream = NULL;
 			g_free (time);
 			audio_view_media_info_next (view);
@@ -792,7 +870,7 @@ audio_view_get_media_info_idler (AudioView *view)
 	}
 	/* release song list */
 	/* FIXME: rather not free the audio info's here, we need them no ? */
-	//eel_g_list_free_deep_custom (song_list, (GFunc) audio_info_free, NULL);
+	/* eel_g_list_free_deep_custom (song_list, (GFunc) audio_info_free, NULL); */
 }
 
 
@@ -826,7 +904,7 @@ audio_view_update (AudioView *view)
 	if (result != GNOME_VFS_OK) {
 		path = (char *) gnome_vfs_get_local_path_from_uri (view->location);
 		message = g_strdup_printf (_("Sorry, but there was an error reading %s."), path);
-		eel_show_error_dialog (message, _("Can't Read Folder"),
+		eel_show_error_dialog (message, _("Can't read folder"),
 				       GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view->event_box))));
 		g_free (path);
 		g_free (message);
@@ -868,14 +946,16 @@ audio_view_update (AudioView *view)
 	/* release song list */
 	/* FIXME: rather not free the audio info's here, we need them no ? */
 	/* so where do we move it to ? */
-	//eel_g_list_free_deep_custom (song_list, (GFunc) audio_info_free, NULL);
+	/*
+	eel_g_list_free_deep_custom (song_list, (GFunc) audio_info_free, NULL);
 
-	//g_free (uri);
+	g_free (uri);
+	*/
 	gtk_widget_show (view->widget);
 	/* create a media info object */
 	view->media_info = gst_media_info_new (NULL);
-	g_print ("DEBUG: adding idler");
-	g_idle_add ((GSourceFunc) audio_view_get_media_info_idler, view);
+	NM_DEBUG("adding idler");
+	view->media_info_idler_id = g_idle_add ((GSourceFunc) audio_view_get_media_info_idler, view);
 }
 
 /* set up the tree view based on the AudioView */
@@ -975,6 +1055,7 @@ audio_view_new ()
 	view = g_new0 (AudioView, 1);
 	/* set to NULL what needs to be NULL */
 	view->error_message = NULL;
+	view->media_info_idler_id = 0;
 
 	/* get a toplevel vbox */
 	view->widget = gtk_vbox_new (FALSE, 5);
@@ -1019,30 +1100,30 @@ audio_view_new ()
 
 	/* prev */
 	view->prev_button = control_box_add_labeled_button (
-			view->control, "Previous", "prev.png");
+			view->control, "Previous", "prev.png", FALSE);
 	g_signal_connect (G_OBJECT (view->prev_button),
 			  "clicked", G_CALLBACK (prev_activate), view);
 	/* stop */
 	view->stop_button = control_box_add_labeled_button (
-			view->control, "Stop", "stop.png");
+			view->control, "Stop", "stop.png", FALSE);
 	g_signal_connect (G_OBJECT (view->stop_button),
 			  "clicked", G_CALLBACK (stop_activate), view);
 
 	/* play */
 	view->play_button = control_box_add_labeled_button (
-			view->control, "Play", "play.png");
+			view->control, "Play", "play.png", FALSE);
 	g_signal_connect (G_OBJECT (view->play_button),
 			  "clicked", G_CALLBACK (play_activate), view);
 
 	/* next */
 	view->next_button = control_box_add_labeled_button (
-			view->control, "Next", "next.png");
+			view->control, "Next", "next.png", FALSE);
 	g_signal_connect (G_OBJECT (view->next_button),
 			  "clicked", G_CALLBACK (next_activate), view);
 
 	/* scan */
 	view->scan_button = control_box_add_labeled_button (
-			view->control, "Scan", "scan.png");
+			view->control, "Scan", "scan.png", TRUE);
 	g_signal_connect (G_OBJECT (view->scan_button),
 			  "clicked", G_CALLBACK (scan_activate), view);
 
@@ -1073,7 +1154,7 @@ audio_view_new ()
 			    view->control, FALSE, FALSE, 0);
 
 	/* get a status area */
-	view->status = gtk_label_new ("Not playing");
+	view->status = gtk_label_new (_("Not playing"));
 	gtk_misc_set_alignment (GTK_MISC (view->status), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (view->widget),
 			    view->status, FALSE, FALSE, 0);
@@ -1120,10 +1201,56 @@ audio_view_new ()
 void
 audio_view_dispose (AudioView *view)
 {
-	g_print ("DEBUG: audio_view_finalize\n");
 	/* no need to destroy widgets, gets done automatically on toplevel */
-	g_print ("DEBUG: audio_play: %p\n", view->audio_play);
+	g_print ("audio_view_dispose started\n");
+	NM_DEBUG("audio_play: %p\n", view->audio_play);
 	/* if it never got created we won't have it will we ? */
-	/* FIXME: maybe fix this in audio_play itself ? */
-	if (view->audio_play) audio_play_dispose (view->audio_play);
+
+	g_print ("unreffing audio object\n");
+	if (view->audio_play) {
+		g_object_unref (view->audio_play);
+		view->audio_play = NULL;
+	}
+
+	if (view->media_info_idler_id) {
+		g_source_remove (view->media_info_idler_id);
+		view->media_info_idler_id = 0;
+	}
+
+	if (view->scan_timeout) {
+		g_source_remove (view->scan_timeout);
+		view->scan_timeout = 0;
+	}
+
+	if (view->media_info) {
+		g_object_unref (view->media_info);
+		view->media_info = NULL;
+	}
+
+	if (view->stream) {
+		g_object_unref (view->stream);
+		view->stream = NULL;
+	}
+}
+
+static void
+free_iter_list (GList *list)
+{
+	GList *l;
+	
+	for (l = list; l != NULL; l = l->next) {
+		gtk_tree_iter_free (l->data);
+	}
+
+	g_list_free (list);
+}
+
+void
+audio_view_finalize (AudioView *view)
+{
+	free_iter_list (view->audio_list);
+
+	g_free (view->error_message);
+	g_free (view->location);
+	g_free (view->selection);
 }
